@@ -2,10 +2,10 @@
  * acct.c	Accounting routines.
  *
  *		Copyright 1996-2001	Cistron Internet Services B.V.
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  */
 char acct_rcsid[] =
-"$Id: acct.c,v 1.20 2002/01/22 14:13:45 miquels Exp $";
+"$Id: acct.c,v 1.26 2003/07/03 08:09:04 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -43,7 +43,7 @@ static char porttypes[] = "ASITX";
  */
 typedef struct nas_port {
 	UINT4			nasaddr;
-	int			port;
+	UINT4			port;
 	off_t			offset;
 	struct nas_port		*next;
 } NAS_PORT;
@@ -53,7 +53,7 @@ static NAS_PORT *nas_port_list = NULL;
 /*
  *	Lookup a NAS_PORT in the nas_port_list
  */
-static NAS_PORT *nas_port_find(UINT4 nasaddr, int port)
+static NAS_PORT *nas_port_find(UINT4 nasaddr, UINT4 port)
 {
 	NAS_PORT	*cl;
 
@@ -119,9 +119,10 @@ char *uue(void *in)
  */
 static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 {
-	char		buf[32];
+	char		buf[48];
 	NAS		*cl;
 	char		*s;
+	int		len;
 
 	/*
 	 *	Fill out the UTMP struct for the radwtmp file.
@@ -143,11 +144,13 @@ static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 		s = cl->shortname;
 	if (s == NULL || s[0] == 0) s = uue(&(ut->nas_address));
 #if UT_LINESIZE > 9
-	sprintf(buf, "%03d:%.20s", ut->nas_port, s);
+	sprintf(buf, "%03u:%.20s", ut->nas_port, s);
 #else
-	sprintf(buf, "%02d%.20s", ut->nas_port, s);
+	sprintf(buf, "%02u%.20s", ut->nas_port, s);
 #endif
-	strncpy(wt->ut_line, buf, UT_LINESIZE);
+	len = UT_LINESIZE;
+	if (len > sizeof(buf)) len = sizeof(buf);
+	strncpy(wt->ut_line, buf, len);
 
 	/*
 	 *	We store the dynamic IP address in the hostname field.
@@ -155,7 +158,9 @@ static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 #ifdef UT_HOSTSIZE
 	if (ut->framed_address) {
 		ipaddr2str(buf, ntohl(ut->framed_address));
-		strncpy(wt->ut_host, buf, UT_HOSTSIZE);
+		len = UT_HOSTSIZE;
+		if (len > sizeof(buf)) len = sizeof(buf);
+		strncpy(wt->ut_host, buf, len);
 	}
 #endif
 #ifdef __svr4__
@@ -183,8 +188,11 @@ static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 
 /*
  *	Zap a user, or all users on a NAS, from the radutmp file.
+ *
+ *	FIXME: add a 'reboot' record to the wtmp file, esp.
+ *	when doutmp == 0 && dowtmp == 1 (and pass in doutmp too).
  */
-int radzap(UINT4 nasaddr, int port, char *user, time_t t, int dowtmp)
+int radzap(UINT4 nasaddr, UINT4 port, char *user, time_t t, int dowtmp)
 {
 	struct radutmp	u;
 	struct utmp	wt;
@@ -213,8 +221,8 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t, int dowtmp)
 		r = 0;
 		while (read(fd, &u, sizeof(u)) == sizeof(u)) {
 			if (((nasaddr != 0 && netaddr != u.nas_address) ||
-			      (port >= 0   && port    != u.nas_port) ||
-			      (user != NULL && strcmp(u.login, user) != 0) ||
+			      (port != (UINT4)-1 && port != u.nas_port) ||
+			      (user != NULL && strncmp(u.login, user, RUT_NAMESIZE) != 0) ||
 			       u.type != P_LOGIN))
 				continue;
 			/*
@@ -247,7 +255,7 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t, int dowtmp)
 /*
  *	Store logins in the RADIUS utmp file.
  */
-int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
+int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp, int doutmp)
 {
 	struct radutmp	ut, u;
 	struct utmp	wt;
@@ -266,6 +274,9 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 	int		nas_port_type = 0;
 	int		off;
 
+	if (!doutmp && !dowtmp)
+		return 1;
+
 	/*
 	 *	Which type is this.
 	 */
@@ -277,6 +288,11 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 	if (status == PW_STATUS_ACCOUNTING_ON ||
 	    status == PW_STATUS_ACCOUNTING_OFF) rb_record = 1;
 
+	if (status == PW_STATUS_ALIVE && !doutmp) {
+		/* Radwtmp alone doesn't care about ALIVE packets. */
+		return 1;
+	}
+
 	if (!rb_record &&
 	    (vp = pairfind(authreq->request, PW_USER_NAME)) == NULL) do {
 		int check1 = 0;
@@ -371,8 +387,8 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 				break;
 			case PW_ACCT_SESSION_ID:
 				/*
-				 *	If length > 8, only store the
-				 *	last 8 bytes.
+				 *	If length > 12, only store the
+				 *	last 12 bytes.
 				 */
 				off = vp->length - sizeof(ut.session_id);
 				/*
@@ -426,13 +442,15 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 	if (status == PW_STATUS_ACCOUNTING_ON && nas_address) {
 		log(L_INFO, "NAS %s restarted (Accounting-On packet seen)",
 			nas_name(nas_address));
-		radzap(nas_address, -1, NULL, ut.time, dowtmp);
+		if (doutmp)
+			radzap(nas_address, -1, NULL, ut.time, dowtmp);
 		return 0;
 	}
 	if (status == PW_STATUS_ACCOUNTING_OFF && nas_address) {
 		log(L_INFO, "NAS %s rebooted (Accounting-Off packet seen)",
 			nas_name(nas_address));
-		radzap(nas_address, -1, NULL, ut.time, dowtmp);
+		if (doutmp)
+			radzap(nas_address, -1, NULL, ut.time, dowtmp);
 		return 0;
 	}
 
@@ -442,7 +460,7 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 	if (status != PW_STATUS_START &&
 	    status != PW_STATUS_STOP &&
 	    status != PW_STATUS_ALIVE) {
-		log(L_ERR, "NAS %s port %d unknown packet type %d)",
+		log(L_ERR, "NAS %s port %u unknown packet type %u)",
 			nas_name(nas_address), ut.nas_port, status);
 		return 0;
 	}
@@ -466,7 +484,8 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 	/*
 	 *	Enter into the radutmp file.
 	 */
-	if ((fd = open(radutmp_path, O_RDWR|O_CREAT, 0644)) >= 0) {
+	fd = 0;
+	if (doutmp && (fd = open(radutmp_path, O_RDWR|O_CREAT, 0644)) >= 0) {
 		NAS_PORT *cache;
 		int r;
 
@@ -501,7 +520,7 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 				 */
 				if (u.type == P_LOGIN)
 					log(L_ERR,
-		"Accounting: logout: entry for NAS %s port %d has wrong ID",
+		"Accounting: logout: entry for NAS %s port %u has wrong ID",
 					nas_name(nas_address), u.nas_port);
 				r = -1;
 				break;
@@ -513,14 +532,14 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 			    u.time >= ut.time) {
 				if (u.type == P_LOGIN) {
 					log(L_INFO,
-		"Accounting: login: entry for NAS %s port %d duplicate",
+		"Accounting: login: entry for NAS %s port %u duplicate",
 					nas_name(nas_address), u.nas_port);
 					r = -1;
 					dowtmp = 0;
 					break;
 				}
 				log(L_ERR,
-		"Accounting: login: entry for NAS %s port %d wrong order",
+		"Accounting: login: entry for NAS %s port %u wrong order",
 				nas_name(nas_address), u.nas_port);
 				r = -1;
 				break;
@@ -575,13 +594,13 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 				write(fd, &u, sizeof(u));
 			} else if (r == 0) {
 				log(L_ERR,
-		"Accounting: logout: login entry for NAS %s port %d not found",
+		"Accounting: logout: login entry for NAS %s port %u not found",
 				nas_name(nas_address), ut.nas_port);
 				r = -1;
 			}
 		}
 		close(fd);
-	} else {
+	} else if (fd < 0) {
 		log(L_ERR, "Accounting: %s: %s", radutmp_path, strerror(errno));
 		ret = -1;
 	}
@@ -816,7 +835,7 @@ int rad_accounting(AUTH_REQ *authreq, int activefd)
 		 *	we run it first, the stripped info will also
 		 *	get into the "detail" file.
 		 */
-		if (rad_accounting_radxtmp(authreq, use_wtmp) == 0)
+		if (rad_accounting_radxtmp(authreq, use_wtmp, use_utmp) == 0)
 			reply = 1;
 		for (i = 0; i < 1 || i < detail_fn_count; i++) {
 			if (rad_accounting_detail(authreq, auth,
@@ -833,7 +852,7 @@ int rad_accounting(AUTH_REQ *authreq, int activefd)
 			    detail_fn[i]) == 0)
 				reply = 1;
 		}
-		if (rad_accounting_radxtmp(authreq, use_wtmp) == 0)
+		if (rad_accounting_radxtmp(authreq, use_wtmp, use_utmp) == 0)
 			reply = 1;
 	}
 
@@ -865,14 +884,15 @@ static void alrm_handler()
  */
 static int rad_check_ts(struct radutmp *ut)
 {
+	NAS	*nas;
+	void	(*handler)(int);
+	char	address[16];
+	char	framed_ip[16];
+	char	session_id[16];
+	char	port[8];
+	char	*s;
 	int	pid, st, e;
 	int	n;
-	NAS	*nas;
-	char	address[16];
-	char	port[8];
-	char	session_id[12];
-	char	*s;
-	void	(*handler)(int);
 
 	/*
 	 *	Find NAS type.
@@ -882,6 +902,10 @@ static int rad_check_ts(struct radutmp *ut)
 		return -1;
 	}
 
+	/* Don't count NASes of type "none" towards total usage. */
+	if (strcmp(nas->nastype, "none") == 0)
+		return 2;
+
 	/*
 	 *	Fork.
 	 */
@@ -926,15 +950,18 @@ static int rad_check_ts(struct radutmp *ut)
 		close(n);
 
 	ipaddr2str(address, ntohl(ut->nas_address));
-	sprintf(port, "%d", ut->nas_port);
-	sprintf(session_id, "%.8s", ut->session_id);
+	framed_ip[0] = 0;
+	if (ut->framed_address)
+		ipaddr2str(framed_ip, ntohl(ut->framed_address));
+	sprintf(port, "%u", ut->nas_port);
+	sprintf(session_id, "%.12s", ut->session_id);
 
 	s = CHECKRAD2;
 	execl(CHECKRAD2, "checkrad", nas->nastype, address, port,
-		ut->login, session_id, NULL);
+		ut->login, session_id, framed_ip, NULL);
 	if (errno == ENOENT) {
 		s = CHECKRAD1;
-		execl(CHECKRAD1, "checklogin", nas->nastype, address, port,
+		execl(CHECKRAD1, "checkrad", nas->nastype, address, port,
 			ut->login, session_id, NULL);
 	}
 	log(L_ERR, "Check-TS: exec %s: %s", s, strerror(errno));
@@ -954,19 +981,35 @@ static int rad_check_ts(struct radutmp *ut)
  *
  *	Returns: 0 == OK, 1 == double logins, 2 == multilink attempt
  */
-int rad_check_multi(char *name, VALUE_PAIR *request, VALUE_PAIR *reply,
+int rad_check_multi(char *name, AUTH_REQ *authreq, VALUE_PAIR *reply,
 	int maxsimul)
 {
 	VALUE_PAIR	*vp;
+	VALUE_PAIR	*request = authreq->request;
 	FILE		*wfp;
+	NAS		*nas;
+	UINT4		ipaddr;
 	struct radutmp	u, u2;
 	struct utmp	wt;
 	char		caller_id[16];
 	int		multi_count = 0;
 	int		portlimit;
 	int		count;
-	int		fd;
+	int		port = -1;
+	int		fd, i;
 
+	/* For NASes of type 'none' don't impose any limits */
+	if ((vp = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL)
+		ipaddr = vp->lvalue;
+	else
+		ipaddr = authreq->ipaddr;
+        if ((nas = nas_find(ipaddr)) != NULL &&
+	    strcmp(nas->nastype, "none") == 0) return 0;
+
+	if ((vp = pairfind(request, PW_CALLING_STATION_ID)) != NULL)
+		strNcpy(caller_id, vp->strvalue, sizeof(caller_id));
+	if ((vp = pairfind(request, PW_NAS_PORT)) != NULL)
+		port = vp->lvalue;
 	if ((fd = open(radutmp_path, O_CREAT|O_RDWR, 0644)) < 0)
 		return 0;
 
@@ -976,8 +1019,15 @@ int rad_check_multi(char *name, VALUE_PAIR *request, VALUE_PAIR *reply,
 	count = 0;
 	while(read(fd, &u, sizeof(u)) == sizeof(u))
 		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
-		    && u.type == P_LOGIN)
+		    && u.type == P_LOGIN) {
+
+			/* Skip duplicates */
+			if (port >= 0 && u.nas_port == port &&
+			    u.nas_address == ipaddr)
+				continue;
+
 			count++;
+		}
 
 	if (count < maxsimul) {
 		close(fd);
@@ -1004,7 +1054,13 @@ int rad_check_multi(char *name, VALUE_PAIR *request, VALUE_PAIR *reply,
 	while (read(fd, &u, sizeof(u)) == sizeof(u)) {
 		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
 		    && u.type == P_LOGIN) {
-			if (rad_check_ts(&u) == 1) {
+
+			/* Skip duplicates */
+			if (port >= 0 && u.nas_port == port &&
+			    u.nas_address == ipaddr)
+				continue;
+
+			if ((i = rad_check_ts(&u)) == 1) {
 				count++;
 				/*
 				 *	Does it look like a MPP attempt?
@@ -1013,7 +1069,7 @@ int rad_check_multi(char *name, VALUE_PAIR *request, VALUE_PAIR *reply,
 				    strcmp(caller_id, u.caller_id) == 0)
 					multi_count++;
 			}
-			else {
+			else if (i <= 0) {
 				/*
 				 *	False record - zap it.
 				 */
