@@ -2,10 +2,10 @@
  * attrprint.c	Functions to print A/V pairs.
  *
  *		Copyright 1998-2001	Cistron Internet Services B.V.
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  */
 char attrprint_rcsid[] =
-"$Id: attrprint.c,v 1.10 2002/01/23 12:42:06 miquels Exp $";
+"$Id: attrprint.c,v 1.14 2003/07/02 09:39:02 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -88,9 +88,9 @@ void fprint_attr_val(FILE *fd, VALUE_PAIR *pair)
 
 	if (pair->flags.has_tag) {
 #ifdef MERIT_TAG_FORMAT
-		fprintf(fd, "%s = :%d:", pair->name,pair->flags.tag);
+		fprintf(fd, "%s = :%u:", pair->name,pair->flags.tag);
 #else
-		fprintf(fd, "%s:%d = ", pair->name,pair->flags.tag);
+		fprintf(fd, "%s:%u = ", pair->name,pair->flags.tag);
 #endif
 	}
 	else {
@@ -138,7 +138,7 @@ void fprint_attr_val(FILE *fd, VALUE_PAIR *pair)
 				memcpy(&type, ptr, 4);
 				left -= 4;
 				i = left;
-				fprintf(fd, ":T%d:L%d:",
+				fprintf(fd, ":T%u:L%d:",
 					(int)ntohl(type), i);
 				ptr += 4;
 			} else
@@ -168,6 +168,16 @@ void fprint_attr_val(FILE *fd, VALUE_PAIR *pair)
 		}
 		break;
 
+	case PW_TYPE_INTEGER8:
+#ifdef USE_LONGLONG
+		fprintf(fd, PERCENT_LONGLONG,
+			((UINT8)pair->lvalueh << 32) | pair->lvalue);
+#else
+		fprintf(fd, "0x%08x%08x", (unsigned int)pair->lvalueh,
+					  (unsigned int)pair->lvalue);
+#endif
+		break;
+
 	case PW_TYPE_IPADDR:
 		if (pair->lvalue == 0 && pair->strvalue[0] != 0) {
 			/* Why is this??? - EvB */
