@@ -2,9 +2,10 @@
  * auth.c	User authentication.
  *
  *		Copyright 1998-2001 Cistron Internet Services B.V.
+ *		Copyright 2002-2003 Cistron IP B.V.
  */
 char auth_rcsid[] =
-"$Id: auth.c,v 1.32 2001/05/26 21:14:49 miquels Exp $";
+"$Id: auth.c,v 1.35 2003/06/04 15:45:27 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/time.h>
@@ -656,7 +657,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		 *	User authenticated O.K. Now we have to check
 		 *	for the Simultaneous-Use parameter.
 		 */
-		if ((r = rad_check_multi(namepair->strvalue, authreq->request,
+		if ((r = rad_check_multi(namepair->strvalue, authreq,
 		    user_reply, check_item->lvalue)) != 0) {
 
 			if (check_item->lvalue > 1) {
@@ -773,6 +774,15 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		auth_item->length = strlen(auth_item->strvalue);
 	}
 
+	if ((auth_item = pairfind(user_reply, PW_RBAK_BIND_SUB_USER_AT_CTX)) != NULL) {
+		ptr = radius_xlate(auth_item->strvalue,
+			authreq->request, user_reply);
+		strncpy(auth_item->strvalue, ptr, sizeof(auth_item->strvalue));
+		auth_item->strvalue[sizeof(auth_item->strvalue) - 1] = 0;
+		auth_item->length = strlen(auth_item->strvalue);
+	}
+
+
 
 	/*
 	 *	If we want to exec a program, but wait for it,
