@@ -17,7 +17,7 @@
  *
  */    
 char *cache_rcsid =
-"$Id: cache.c,v 1.7 2001/05/07 20:33:45 miquels Exp $";
+"$Id: cache.c,v 1.8 2003/07/02 09:39:03 miquels Exp $";
 
 #include<stdio.h>
 #include<string.h>
@@ -101,7 +101,7 @@ int buildHashTable(void) {
 
 			/* Hash the username */
 			hashindex = hashUserName(username);	
-			/*printf("%s:%d\n", username, hashindex);*/
+			/*printf("%s:%u\n", username, hashindex);*/
 	
 			/* Allocate space for structure to go in hashtable */
 			if((new = (struct mypasswd *)malloc(sizeof(struct mypasswd))) == NULL) {
@@ -403,13 +403,16 @@ int hashradutmp(void) {
 	int fd;
 	struct radutmp u;
 	struct mypasswd *cur;
+	char tmp[RUT_NAMESIZE+1];
 
    if ((fd = open(radutmp_path, O_CREAT|O_RDONLY, 0644)) < 0)
       return 0;
 
    while(read(fd, &u, sizeof(u)) == sizeof(u)) {
-      if ((u.login) && (u.type == P_LOGIN)) {
-			cur = findHashUser(u.login);
+      if ((u.login[0]) && (u.type == P_LOGIN)) {
+			tmp[0] = 0;
+			strncat(tmp, u.login, RUT_NAMESIZE);
+			cur = findHashUser(tmp);
 			if(cur) {
 				cur->loggedin++;		
 			}
