@@ -10,23 +10,29 @@
 #
 # Returns:	0 = no duplicate, 1 = duplicate, >1 = error.
 #
-# Version:	@(#)checkrad.pl  1.18  25-Jul-1999  miquels@cistron.nl
+# Version:	@(#)checkrad.pl  1.31  22-Mar-2001  miquels@cistron.nl
 #
-#		livingston_snmp  1.1	Author: miquels@cistron.nl
+#		livingston_snmp  1.2	Author: miquels@cistron.nl
+#		cvx_snmp	 1.0	Author: miquels@cistron.nl
 #		portslave_finger 1.0	Author: miquels@cistron.nl
 #		max40xx_finger	 1.0	Author: costa@mdi.ca
-#		computone_finger 1.1	Author: costa@mdi.ca
+#		ascend_snmp      1.1    Author: blaz@amis.net
+#		computone_finger 1.2	Author: pacman-radius@cqc.com
 #		sub tc_tccheck	 1.1	Author: alexisv@compass.com.ph
-#		pathras_telnet	 1.1	Author: accdias@sst.com.br
+#		cyclades_telnet	 1.2	Author: accdias@sst.com.br
 #		patton_snmp	 1.0	Author: accdias@sst.com.br
-#		digitro_rusers	 1.0	Author: accdias@sst.com.br
+#		digitro_rusers	 1.1	Author: accdias@sst.com.br
+#		cyclades_snmp	 1.1	Author: accdias@sst.com.br
 #		usrhiper_snmp    1.0    Author: igor@ipass.net
 #		multitech_snmp   1.0    Author: ehonzay@willmar.com
 #		netserver_telnet 1.0	Author: mts@interplanet.es
 #		versanet_snmp    1.0    Author: support@versanetcomm.com
+#		bay_finger	 1.0	Author: chris@shenton.org
 #
 #	Config: $debug is the file you want to put debug messages in
 #		$snmpget is the location of your ``snmpget'' program
+#		$snmpwalk is the location of your ``snmpwalk'' program
+#		$rusers is the location of your ``rusers'' program
 #		$naspass is the location of your NAS admin password file
 #
 
@@ -40,16 +46,15 @@ $debug		= "";
 #$debug		= "$logdir/checkrad.log";
 
 $snmpget	= "/usr/bin/snmpget";
-$naspass	= "$raddbdir/naspasswd";
+$snmpwalk	= "/usr/bin/snmpwalk";
 $rusers		= "/usr/bin/rusers";
+$naspass	= "$raddbdir/naspasswd";
 
 #
-#	PM3:	$lv_offs is where the last S port is before one or two
-#		ports are skipped (22 or 29, for US or Europe)
-#		$lv_hole is the size of the hole (1 or 2, for US or Europe).
+#	USR-Hiper: $hiper_density is the reported port density (default 256
+#	but 24 makes more sense)
 #
-$lv_offs = 29;
-$lv_hole = 2;
+$hiper_density = 256;
 
 #
 #	Try to load Net::Telnet, SNMP_Session etc.
@@ -83,7 +88,7 @@ sub naspasswd {
 		if (!$emptyok) {
 			print LOG "checkrad: naspasswd file not found; " .
 			"possible match for $ARGV[3]\n" if ($debug);
-			print STDERR "checkrad: naspassswd file not found; " .
+			print STDERR "checkrad: naspasswd file not found; " .
 			"possible match for $ARGV[3]\n";
 		}
 		return ();
@@ -237,39 +242,112 @@ sub strip_username {
 #
 $lvm	 = '.iso.org.dod.internet.private.enterprises.307';
 sub livingston_snmp {
-
 	#
-	#	First find out the offset (ugly!!). Also, if the portno
-	#	is greater than 29, substract 2 (S30 and S31 don't exist).
-	#	You might need to change this to 23 and 1 for the USA.
+	#	We don't know at which ifIndex S0 is, and
+	#	there might be a hole at S23, or at S30+S31.
+	#	So we figure out dynamically which offset to use.
 	#
-	$_ = snmpget($ARGV[1], "public", "$lvm.3.2.1.1.1.2.5");
+	#	If the port < S23, probe ifIndex 5.
+	#	If the port < S30, probe IfIndex 23.
+	#	Otherwise probe ifIndex 32.
+	#
+	my $ifIndex;
+	my $test_index;
+	if ($ARGV[2] < 23) {
+		$test_index = 5;
+	} elsif ($ARGV[2] < 30) {
+		$test_index = 23;
+	} else {
+		$test_index = 32;
+	}
+	$_ = snmpget($ARGV[1], "public", "$lvm.3.2.1.1.1.2.$test_index");
 	/S([0-9]+)/;
 	$xport = $1 + 0;
-	$portidx = $ARGV[2] + (5 - $xport);
-	$portidx -= $lv_hole if ($ARGV[2] > $lv_offs);
-	chop;
-	print LOG "  using $xport offset for port / SNMPno translation\n"
+	$ifIndex = $ARGV[2] + ($test_index - $xport);
+
+	print LOG "  port S$ARGV[2] at SNMP ifIndex $ifIndex\n"
 		if ($debug);
 
 	#
 	#	Now get the session id from the terminal server.
 	#
-	$sessid = snmpget($ARGV[1], "public", "$lvm.3.2.1.1.1.5.$portidx");
+	$sessid = snmpget($ARGV[1], "public", "$lvm.3.2.1.1.1.5.$ifIndex");
 
 	print LOG "  session id at port S$ARGV[2]: $sessid\n" if ($debug);
 
 	($sessid eq $ARGV[4]) ? 1 : 0;
 }
 
+#
+#	See if the user is logged in using the Aptis MIB.
+#	We don't check the username but the session ID.
+#
+# sessionStatusActiveName
+$apm1	 = '.iso.org.dod.internet.private.enterprises.2637.2.2.102.1.12';
+# sessionStatusActiveStopTime
+$apm2	 = '.iso.org.dod.internet.private.enterprises.2637.2.2.102.1.20';
+sub cvx_snmp {
+
+	# Remove unique identifier, then take remainder of the
+	# session-id as a hex number, convert that to decimal.
+	my $sessid = $ARGV[4];
+	$sessid =~ s/^.*://;
+	$sessid =~ s/^0*//;
+	$sessid = "0" if ($sessid eq '');
+
+	#
+	#	Now get the login from the terminal server.
+	#	Blech - the SNMP table is called 'sessionStatusActiveTable,
+	#	but it sometimes lists inactive sessions too.
+	#	However an active session doesn't have a Stop time,
+	#	so we can differentiate that way.
+	#
+	my $login = snmpget($ARGV[1], "public", "$apm1." . hex($sessid));
+	my $stopt = snmpget($ARGV[1], "public", "$apm2." . hex($sessid));
+	$login = "--" if ($stopt > 0);
+
+	print LOG "  login with session-id $ARGV[4]: $login\n" if ($debug);
+
+	(strip_username($login) eq strip_username($ARGV[3])) ? 1 : 0;
+}
+
 #
 #	See if the user is logged in using the Cisco MIB
 #
 $csm	 = '.iso.org.dod.internet.private.enterprises.9';
 sub cisco_snmp {
-	$login = snmpget($ARGV[1], "public", "$csm.2.9.2.1.18.$ARGV[2]");
 
-	print LOG "  user at port S$ARGV[2]: $login\n" if ($debug);
+	# Look up community string in naspasswd file.
+	my ($login, $pass) = naspasswd($ARGV[1], 1);
+	if ($login eq '') {
+		$pass = 'public';
+	} elsif ($login ne 'SNMP') {
+		if ($debug) {
+			print LOG
+			"   Error: Need SNMP community string for $ARGV[1]\n";
+		}
+		return 2;
+	}
+
+	my $port = $ARGV[2];
+	my $sess_id = hex($ARGV[4]);
+
+	if ($port < 20000) {
+		#
+		#	The AS5350 doesn't support polling the session ID,
+		#	so we do it based on nas-port-id. This only works
+		#	for analog sessions where port < 20000.
+		#	Yes, this means that simultaneous-use on the as5350
+		#	doesn't work for ISDN users.
+		#
+		$login = snmpget($ARGV[1], $pass, "$csm.2.9.2.1.18.$port");
+		print LOG "  user at port S$port: $login\n" if ($debug);
+	} else {
+		$login = snmpget($ARGV[1], $pass,
+				"$csm.9.150.1.1.3.1.2.$sess_id");
+		print LOG "  user with session id $ARGV[4] ($sess_id): " .
+			"$login\n" if ($debug);
+	}
 
 	($login eq $ARGV[3]) ? 1 : 0;
 }
@@ -290,25 +368,55 @@ sub multitech_snmp {
 }
 
 #
-#	Check a Computone Powerrack via finger
+#       Check a Computone Powerrack via finger
 #
-#	Author: Shiloh Costa of MDI Internet Inc. <costa@mdi.ca>
+#       Old Author: Shiloh Costa of MDI Internet Inc. <costa@mdi.ca>
+#       New Author: Alan Curry of ComQuest Internet Services. <pacman@cqc.com>
+#
+# The finger response format is version-dependent. To do this *right*, you
+# need to know exactly where the port number and username are. I know that
+# for 1.7.2, and 3.0.4 but for others I just guess.
+# Oh yeah and on top of it all, the thing truncates usernames. --Pac.
+#
+# 1.7.2 and 3.0.4 both look like this:
+#
+# 0    0 000 00:56 luser         pppfsm  Incoming PPP, ppp00, 10.0.0.1
+#
+# and the truncated ones look like this:
+#
+# 25   0 000 00:15 longnameluse..pppfsm  Incoming PPP, ppp25, 10.0.0.26
+#
+# Yes, the fields run together. Long Usernames Considered Harmful.
 #
 sub computone_finger {
-	$online = 0;
-	open(FD, "finger \@$ARGV[1]|");
-	while(<FD>) {  
-	   @line = <FD>;
-	   }
-	 $numline = @line;
-	 for ($num = 0; $num < $numline; $num++) {
-	   if( $line[$num]  =~ / $ARGV[3] / ){
-	     #print "$ARGV[3] is online\n";
-	     $online = 1; # user is online
-	   }
-	 }
+	my $trunc, $ver;
+
+	open(FD, "finger \@$ARGV[1]|") or return 2;
+	<FD>; # the [hostname] line is definitely uninteresting
+	$trunc = substr($ARGV[3], 0, 12);
+	$ver = "";
+	while(<FD>) {
+		if(/cnx kernel release ([^ ,]+)[, ]/) {
+			$ver = $1;
+			next;
+		}
+		# Check for known versions
+		if ($ver eq '1.7.2' || $ver eq '3.0.4') {
+			if (/^\Q$ARGV[2]\E\s+\S+\s+\S+\s+\S+\s+\Q$trunc\E(\s+|\.\.)/) {
+	 			close FD;
+				return 1;
+			}
+			next;
+		}
+		# All others.
+		if (/^\s*\Q$ARGV[2]\E\s+.*\s+\Q$trunc\E\s+/) {
+			close FD;
+			return 1;
+		}
+	}
+
 	close FD;
-	return $online;
+	return 0;
 }
 
 #
@@ -332,6 +440,38 @@ sub max40xx_finger {
 	close FD;
 }
 
+
+#
+#	Check an Ascend Max4000 or similar model via SNMP
+#
+#	Author: Blaz Zupan of Medinet <blaz@amis.net>
+#
+$asm   = '.iso.org.dod.internet.private.enterprises.529';
+sub ascend_snmp {
+	my $sess_id;
+	my $l1, $l2;
+
+	$l1 = '';
+	$l2 = '';
+
+	#
+	#	If it looks like hex, only try it as hex,
+	#	otherwise try it as both decimal and hex.
+	#
+	$sess_id = $ARGV[4];
+	if ($sess_id !~ /^0/ && $sess_id !~ /[a-f]/i) {
+		$l1 = snmpget($ARGV[1], "public", "$asm.12.3.1.4.$sess_id");
+	}
+	$sess_id = hex $ARGV[4];
+	$l2 = snmpget($ARGV[1], "public", "$asm.12.3.1.4.$sess_id");
+
+	print LOG "  user at port S$ARGV[2]: $l1 (dec)\n" if ($debug && $l1);
+	print LOG "  user at port S$ARGV[2]: $l2 (hex)\n" if ($debug && $l2);
+
+	(($l1 && $l1 eq $ARGV[3]) || ($l2 && $l2 eq $ARGV[3])) ? 1 : 0;
+}
+
+
 #
 #	See if the user is logged in using the portslave finger.
 #
@@ -505,117 +645,127 @@ sub tc_tccheck {
 #
 #	Check a Cyclades PathRAS via telnet
 #
-#	Version: 1.1
+#	Version: 1.2
 #
 #	Author: Antonio Dias of SST Internet <accdias@sst.com.br>
 #
-sub pathras_telnet {
-   #
-   #	Localize all variables first.
-   #
-   my (@users, $pathras, $port, $errmsg, $user, $login, $password);
-   #
-   #	Do we have Net::Telnet installed?
-   #
-   return 2 unless (check_net_telnet());
-   #
-   #	Get login name and password for NAS from $naspass file.
-   #
-   ($login, $password) = naspasswd($ARGV[1], 1);
-   #
-   #	NAS doesn't have a password defined
-   #
-   return 2 if ($password eq "");
-   #
-   #	Communicate with PathRAS using Net::Telnet, then access
-   #	menu option 6.8 to see who are logged in. This assumes 
-   #	you are using the default Prompt String (menu option 6.2)
-   #	and is based on PathRAS firmware version 1.1.9g
-   # 
-   #
-   $pathras = new Net::Telnet (
-   	Timeout => 10, 
-   	Prompt => '/(.*)\s:*$/', 
-   	Errmode => 'return'
-   );
-   #
-   #
-   #
-   if (!$pathras->open($ARGV[1])) {;
-      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
-      $pathras->close;
-      return 2;
-   }
-   #
-   #	Dunno why but PathRAS only issue its banner and 
-   #	the login prompt after receive a "CR" 
-   #
-   if (!$pathras->print("")) {
-      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
-      $pathras->close;
-      return 2;
+sub cyclades_telnet {
+	#
+	#	Localize all variables first.
+	#
+	my ($pr, $pr_login, $pr_passwd, $pr_prompt, $endlist, @list, $port, $user);
+	#
+	#	This variable must match PathRAS' command prompt 
+	#	string as entered in menu option 6.2.
+	#	The value below matches the default command prompt.
+	#
+	$pr_prompt = '/Select option ==\>$/i';
 
-   }
-   #
-   #	Log on PathRAS
-   #
-   if (!($pathras->login($login, $password))) {
-      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
-      $pathras->close;
-      return 2;
-   }
-   #
-   #  Access menu option 6 "PathRAS Management"
-   #
-   if (!($pathras->cmd(String => '6', 
-   	Prompt => '/ ==\> $/', 
-   	Timeout => 10))) {
-      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
-      $pathras->close;
-      return 2;
-   }
-   #
-   #  Access menu option 8 "Show Active Ports"
-   #
-   if (!(@users = $pathras->cmd(String => '8', 
-   	Prompt => '/Type \<enter\>/', 
-   	Timeout => 10))) {
-      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
-      $pathras->close;
-      return 2;
-   }
-   #
-   # Since we got the info we want, let's close the session
-   #
-   $pathras->close;
-   #
-   #	Telnet closed.  We got the info so let's examine it.
-   #
-   foreach(@users) {
-      if (m/PPP Active|SLIP Active/) {
-         ($port, $user) = split;
-         #
-         # Strip out any prefixes and suffixes 
-         # from the username
-         #
-         $user = strip_username($user);
-         #
-         # Check if $user is already connected
-         #
-         if ($user eq $ARGV[3]) {
-            print LOG "  User $ARGV[3] found on " . 
-               "NAS $ARGV[1] port $port.\n" if ($debug);
-            return 1;
-         }
-      }
-   }
-   print LOG "  User $ARGV[3] not found on " .
-   	"NAS $ARGV[1].\n" if ($debug);
-   0;
+	#
+	#	This variable match the end of userslist.
+	#
+	$endlist = '/Type \<enter\>/i';
+
+	#
+	#	Do we have Net::Telnet installed?
+	#
+	return 2 unless (check_net_telnet());
+
+	#
+	#	Get login name and password for NAS 
+	#	from $naspass file.
+	#
+	($pr_login, $pr_passwd) = naspasswd($ARGV[1], 1);
+
+	#
+	#	Communicate with PathRAS using Net::Telnet, then access
+	#	menu option 6.8 to see who are logged in.
+	#	Based on PathRAS firmware version 1.2.3
+	#
+	$pr = new Net::Telnet (
+		Timeout		=> 10,
+		Host		=> $ARGV[1],
+		ErrMode		=> 'return'
+	) || return 2;
+
+	#
+	#	Force PathRAS shows its banner.
+	#
+	$pr->break();
+
+	#
+	#	Log on PathRAS
+	#
+	if ($pr->waitfor(Match => '/login : $/i') == 1) {
+		$pr->print($pr_login);
+	} else { 
+		print LOG " Error: sending login name to PathRAS\n" if ($debug);
+		$pr->close;
+		return 2;	
+	}
+
+	if ($pr->waitfor(Match => '/password : $/i') == 1) {
+		$pr->print($pr_passwd);
+	} else { 
+		print LOG " Error: sending password to PathRAS.\n" if ($debug);
+		$pr->close;
+		return 2;	
+	}
+
+	$pr->print();
+
+	#
+	#	Access menu option 6 "PathRAS Management"
+	#
+	if ($pr->waitfor(Match => $pr_prompt) == 1) {
+		$pr->print('6');
+	} else { 
+		print LOG "  Error: acessing menu option '6'.\n" if ($debug);
+		$pr->close;
+		return 2;
+	}
+	#
+	#	Access menu option 8 "Show Active Ports"
+	#
+	if ($pr->waitfor(Match => $pr_prompt) == 1) {
+		@list = $pr->cmd(String => '8', Prompt => $endlist);
+	} else { 
+		print LOG "  Error: acessing menu option '8'.\n" if ($debug);
+		$pr->close;
+		return 2;
+	}
+	#
+	#	Since we got the info we want, let's close 
+	#	the telnet session
+	#
+	$pr->close;
+
+	#
+	#	Lets examine the userlist stored in @list
+	#
+	foreach(@list) {
+		#
+		#	We are interested in active sessions only
+		#
+		if (/Active/i) {
+			($port, $user) = split;
+			#
+			#	Strip out any prefix, suffix and
+			#	realm from $user check to see if
+			#	$ARGV[3] matches.
+			#
+			if($ARGV[3] eq strip_username($user)) {
+				print LOG "  User '$ARGV[3]' found on '$ARGV[1]:$port'.\n" if ($debug);
+				return 1;
+			}
+		}
+	}
+	print LOG "  User '$ARGV[3]' not found on '$ARGV[1]'.\n" if ($debug);
+   	0;
 }
 
 #
-#	Check a Patton 2800 via snmp
+#	Check a Patton 2800 via SNMP
 #
 #	Version: 1.0
 #
@@ -624,7 +774,10 @@ sub pathras_telnet {
 sub patton_snmp {
    my($oid);
 
-   $oid = '.1.3.6.1.4.1.1768.5.100.1.40.' . hex $ARGV[4];
+   #$oid = '.1.3.6.1.4.1.1768.5.100.1.40.' . hex $ARGV[4];
+   # Reported by "Andria Legon" <andria@patton.com>
+   # The OID below should be the correct one instead of the one above.
+   $oid = '.1.3.6.1.4.1.1768.5.100.1.56.' . hex $ARGV[4];
    #
    # Check if the session still active
    # 
@@ -639,26 +792,54 @@ sub patton_snmp {
 #
 #      Check a Digitro BXS via rusers
 #
-#      Version: 1.0
+#      Version: 1.1
 #
 #      Author: Antonio Dias of SST Internet <accdias@sst.com.br>
 #
 sub digitro_rusers {
+   my ($ret);
+   local $_;
+
    if (-e $rusers && -x $rusers) {
-      open(FD, "$rusers $ARGV[1]|");
+      #
+      # Get a list of users logged in via rusers
+      #
+      $_ = `$rusers $ARGV[1]`;
+      $ret = ((/$ARGV[3]/) ? 1 : 0);
    } else {
       print LOG "   Error: can't execute $rusers\n" if $debug;
-      return 2;
+      $ret = 2;
    }
+   $ret;
+}
 
-   while(<FD>) {
-      if (/$ARGV[3]/) {
-	close FD;
-	return 1;
-      }
+#
+#	Check Cyclades PR3000 and PR4000 via SNMP
+#
+#	Version: 1.0
+#
+#	Author: Antonio Dias of SST Internet <accdias@sst.com.br>
+#
+sub cyclades_snmp {
+   my ($oid, $ret);
+   local $_;
+   
+   $oid = ".1.3.6.1.4.1.2925.3.3.6.1.1.2";
+
+   if (-e $snmpwalk && -x $snmpwalk) { 	
+      #
+      # Get a list of logged in users via snmpwalk
+      #
+      # I'm looking for a way to do this using SNMP_Session
+      # module. Any help would be great.
+      #
+      $_ = `$snmpwalk $ARGV[1] public $oid`;
+      $ret = ((/"$ARGV[3]"/) ? 1 : 0);
+   } else {
+      print LOG "   Error: can't execute $snmpwalk\n" if $debug;
+      $ret = 2;
    }
-   close FD;
-   0;
+   $ret;
 }
 
 #
@@ -668,13 +849,29 @@ sub digitro_rusers {
 # 
 $usrm	 = '.iso.org.dod.internet.private.enterprises.429';
 sub usrhiper_snmp {
+	my ($login,$password,$oidext);
 
-	# Somebody please verify????
-	my($oidext) = $ARGV[2] + 1256;
+	# Look up community string in naspasswd file.
+        ($login, $password) = naspasswd($ARGV[1], 1);
+	if ($login eq '') {
+		$pass = 'public';
+	} elsif ($login ne 'SNMP') {
+		if ($debug) {
+			print LOG
+			"   Error: Need SNMP community string for $ARGV[1]\n";
+		}
+		return 2;
+	}
+
+	$oidext = 1257 + 256*int(($ARGV[2]-1) / $hiper_density) +
+		                (($ARGV[2]-1) % $hiper_density);
 	my ($login);
 
-	$login = snmpget($ARGV[1], "public", "$usrm.4.10.1.1.18.$oidext");
-	($login) = /^.*\"([^"]+)".*$/;
+	$login = snmpget($ARGV[1], $password, "$usrm.4.10.1.1.18.$oidext");
+	if ($login =~ /\"/) {
+		$login =~ /^.*\"([^"]+)\"/;
+		$login = $1;
+	}
 
 	print LOG "  user at port S$ARGV[2]: $login\n" if ($debug);
 
@@ -787,17 +984,61 @@ sub versanet_snmp {
 }
 
 
-if ($debug) {
-	open(LOG, ">>$debug");
-	$now = localtime;
-	print LOG "$now checkrad @ARGV\n";
+# 1999/08/24 Chris Shenton <chris@shenton.org>
+# Check Bay8000 NAS (aka: Annex) using finger. 
+# Returns from "finger @bay" like:
+#   Port  What User         Location         When          Idle  Address
+#   asy2  PPP  bill         ---              9:33am         :08  192.168.1.194
+#   asy4  PPP  hillary      ---              9:36am         :04  192.168.1.195
+#   [...]
+# But also returns partial-match users if you say like "finger g@bay":
+#   Port  What User         Location         When          Idle  Address
+#   asy2  PPP  gore         ---              9:33am         :09  192.168.1.194
+#   asy22 PPP  gwbush       ---              Mon  9:19am    :07  192.168.1.80
+# So check exact match of username!
+
+sub bay_finger {		# ARGV: 1=nas_ip, 2=nas_port, 3=login, 4=sessid
+    open(FINGER, "finger $ARGV[3]\@$ARGV[1]|") || return 2; # error
+    while(<FINGER>) {
+	my ($Asy, $PPP, $User) = split;
+	if( $User =~ /^$ARGV[3]$/ ){
+	    close FINGER;
+	    print LOG "checkrad:bay_finger: ONLINE $ARGV[3]\@$ARGV[1]"
+		if ($debug);
+	    return 1; # online
+	}
+    }
+    close FINGER;
+    print LOG "checkrad:bay_finger: offline $ARGV[3]\@$ARGV[1]" if ($debug);
+    return 0; # offline
 }
 
-if ($#ARGV != 4) {
-	print LOG "Usage: checkrad nas_type nas_ip " .
-			"nas_port login session_id\n" if ($debug);
-	print STDERR "Usage: checkrad nas_type nas_ip " .
-			"nas_port login session_id\n";
+###############################################################################
+
+# Poor man's getopt (for -d)
+if ($ARGV[0] eq '-d') {
+	shift @ARGV;
+	$debug = "stdout";
+}
+
+if ($debug) {
+	if ($debug eq 'stdout') {
+		open(LOG, ">&STDOUT");
+	} elsif ($debug eq 'stderr') {
+		open(LOG, ">&STDERR");
+	} else {
+		open(LOG, ">>$debug");
+		$now = localtime;
+		print LOG "$now checkrad @ARGV\n";
+	}
+}
+
+if ($#ARGV < 4) {
+	print LOG "Usage: checkrad [-d] nas_type nas_ip " .
+			"nas_port login session_id [framed_ip]\n" if ($debug);
+	print STDERR "Usage: checkrad [-d] nas_type nas_ip " .
+			"nas_port login session_id [framed_ip]\n"
+			unless ($debug =~ m/^(stdout|stderr)$/);
 	close LOG if ($debug);
 	exit(2);
 }
@@ -806,18 +1047,26 @@ if ($ARGV[0] eq 'livingston') {
 	$ret = &livingston_snmp;
 } elsif ($ARGV[0] eq 'cisco') {
 	$ret = &cisco_snmp;
+} elsif ($ARGV[0] eq 'cvx') {
+	$ret = &cvx_snmp;
 } elsif ($ARGV[0] eq 'multitech') {
         $ret = &multitech_snmp;
 } elsif ($ARGV[0] eq 'computone') {
 	$ret = &computone_finger;
 } elsif ($ARGV[0] eq 'max40xx') {
 	$ret = &max40xx_finger;
+} elsif ($ARGV[0] eq 'ascend' || $ARGV[0] eq 'max40xx_snmp') {
+	$ret = &ascend_snmp;
 } elsif ($ARGV[0] eq 'portslave') {
 	$ret = &portslave_finger;
 } elsif ($ARGV[0] eq 'tc') {
 	$ret = &tc_tccheck;
 } elsif ($ARGV[0] eq 'pathras') {
-	$ret = &pathras_telnet;
+	$ret = &cyclades_telnet;
+} elsif ($ARGV[0] eq 'pr3000') {
+	$ret = &cyclades_snmp;
+} elsif ($ARGV[0] eq 'pr4000') {
+	$ret = &cyclades_snmp;
 } elsif ($ARGV[0] eq 'patton') {
 	$ret = &patton_snmp;
 } elsif ($ARGV[0] eq 'digitro') {
@@ -826,8 +1075,10 @@ if ($ARGV[0] eq 'livingston') {
 	$ret = &usrhiper_snmp;
 } elsif ($ARGV[0] eq 'netserver') {
 	$ret = &usrnet_telnet;
-} elsif ($ARGV[0] eq 'versanet'){
+} elsif ($ARGV[0] eq 'versanet') {
         $ret = &versanet_snmp;
+} elsif ($ARGV[0] eq 'bay') {
+	$ret = &bay_finger;
 } elsif ($ARGV[0] eq 'other') {
 	$ret = 1;
 } else {
