@@ -3,10 +3,10 @@
  *
  *		Copyright 1998-2001	Cistron Internet Services B.V.
  *		Copyright 2000		Emile van Bergen (tagged attributes)
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  */
 char dict_rcsid[] =
-"$Id: dict.c,v 1.9 2002/01/23 12:42:06 miquels Exp $";
+"$Id: dict.c,v 1.11 2003/06/04 15:45:27 miquels Exp $";
 
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -160,6 +160,9 @@ static DICT_ATTR *parse_attr(char *buffer, int line_no, char *dict_file,
 	else if(strcmp(typestr, "integer") == 0) {
 		type = PW_TYPE_INTEGER;
 	}
+	else if(strcmp(typestr, "integer8") == 0) {
+		type = PW_TYPE_INTEGER8;
+	}
 	else if(strcmp(typestr, "ipaddr") == 0) {
 		type = PW_TYPE_IPADDR;
 	}
