@@ -2,10 +2,11 @@
  * exec.c	Execute external programs.
  *
  *		Copyright 1998,2001 Cistron Internet Services B.V.
+ *		Copyright 2002-2003 Cistron IP B.V.
  *
  */
 char exec_rcsid[] =
-"$Id: exec.c,v 1.11 2001/11/26 21:51:59 miquels Exp $";
+"$Id: exec.c,v 1.13 2003/07/02 09:39:03 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/time.h>
@@ -40,7 +41,8 @@ char exec_rcsid[] =
 char *radius_xlate(char *str, VALUE_PAIR *request, VALUE_PAIR *reply)
 {
 	static char buf[2048];
-	int n, i = 0, c;
+	unsigned int n;
+	int i = 0, c;
 	char *p;
 	VALUE_PAIR *tmp;
 
@@ -81,7 +83,7 @@ char *radius_xlate(char *str, VALUE_PAIR *request, VALUE_PAIR *reply)
 				     PW_FRAMED_MTU)) != NULL) {
 					n = tmp->lvalue;
 				}
-				sprintf(buf + i, "%d", n);
+				sprintf(buf + i, "%u", n);
 				i += strlen(buf + i);
 				break;
 			case 'p': /* Port number */
@@ -90,7 +92,7 @@ char *radius_xlate(char *str, VALUE_PAIR *request, VALUE_PAIR *reply)
 				     PW_NAS_PORT)) != NULL) {
 					n = tmp->lvalue;
 				}
-				sprintf(buf + i, "%d", n);
+				sprintf(buf + i, "%u", n);
 				i += strlen(buf + i);
 				break;
 			case 'u': /* User name */
