@@ -4,11 +4,11 @@
  *		and to lookup values.
  *
  *		Copyright 1996-2001	Cistron Internet Services B.V.
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  */
 
 char files_rcsid[] =
-"$Id: files.c,v 1.40 2002/02/06 15:23:16 miquels Exp $";
+"$Id: files.c,v 1.44 2003/06/04 15:45:28 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -385,6 +385,14 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 						 check_item->strvalue);
 				break;
 
+			case PW_TYPE_INTEGER8:
+				compare = auth_item->lvalueh -
+					check_item->lvalueh;
+				if (compare == 0)
+					compare = auth_item->lvalue -
+						check_item->lvalue;
+				break;
+
 			case PW_TYPE_INTEGER:
 				if (check_item->attribute == PW_NAS_PORT &&
 				    check_item->operator == PW_OPERATOR_EQUAL) {
@@ -511,6 +519,13 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 				}
 				break;
 
+			case PW_TYPE_INTEGER8:
+				if(check_item->lvalueh == auth_item->lvalueh &&
+				   check_item->lvalue == auth_item->lvalue) {
+					result = 0;
+				}
+				break;
+
 			case PW_TYPE_INTEGER:
 				if (check_item->attribute == PW_NAS_PORT) {
 					if (portcmp(check_item, auth_item) == 0)
@@ -1617,16 +1632,22 @@ int read_realms_file(char *file)
  *	Find a realm in the REALM list. If the realm can't be found, use
  *	the DEFAULT realm, *unless* the realm was the NULL realm.
  */
-REALM *realm_find(char *r, int dfl)
+REALM *realm_find(char *r, char *n, int dfl)
 {
 	REALM	*cl;
 
 	/* Sanity check */
 	if (r == NULL || r[0] == 0) return NULL;
 
-	for(cl = realms; cl; cl = cl->next)
-		if (strcmp(cl->realm, r) == 0)
-			break;
+	for(cl = realms; cl; cl = cl->next) {
+		if (cl->opts.wildcard && n != NULL) {
+			if (wildmat(cl->realm, n))
+				break;
+		} else {
+			if (strcmp(cl->realm, r) == 0)
+				break;
+		}
+	}
 	if (cl || !dfl || !strcmp(r, "NULL")) return cl;
 
 	for(cl = realms; cl; cl = cl->next)
