@@ -49,12 +49,14 @@ typedef struct pw_auth_hdr {
 #define PW_ACCT_UDP_PORT		1646
 
 #define VENDORPEC_USR			429
+#define VENDORPEC_REDBACK		2352
 #define VENDORPEC_CISTRON		8246
 
 #define PW_TYPE_STRING			0
 #define PW_TYPE_INTEGER			1
 #define PW_TYPE_IPADDR			2
 #define PW_TYPE_DATE			3
+#define PW_TYPE_INTEGER8		4
 
 
 #define	PW_AUTHENTICATION_REQUEST	1
@@ -120,6 +122,7 @@ typedef struct pw_auth_hdr {
 
 /* Vendor specific attributes */
 #define PW_CISTRON_PROXIED_TO		((VENDORPEC_CISTRON<<16)|11)
+#define PW_RBAK_BIND_SUB_USER_AT_CTX	((VENDORPEC_REDBACK<<16)|58)
 
 /* Server side attributes */
 #define PW_FALL_THROUGH			500
