@@ -2,12 +2,12 @@
  * radiusd.c	Main loop of the server.
  *
  *		Copyright 1996-2001	Cistron Internet Services B.V.
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  */
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 char radiusd_rcsid[] =
-"$Id: radiusd.c,v 1.24 2002/01/23 12:42:07 miquels Exp $";
+"$Id: radiusd.c,v 1.29 2003/07/02 09:39:03 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -48,6 +48,7 @@ int			log_stripped_names;
 int 			cache_passwd = 0;
 int			use_dbm = 0;
 int			use_wtmp = 1;
+int			use_utmp = 1;
 UINT4			myip = 0;
 UINT4			warning_seconds;
 char			*log_auth_detail;
@@ -57,6 +58,8 @@ int			log_auth_goodpass  = 0;
 int			auth_port;
 int			acct_port;
 
+extern int		optind;
+
 /*
  *	Make sure recv_buffer is aligned properly.
  */
@@ -284,7 +287,10 @@ int main(int argc, char **argv)
 			break;
 
 		case 'u':
-			radutmp_path = optarg;
+			if (strcmp(optarg, "none") == 0)
+				use_utmp = 0;
+			else
+				radutmp_path = optarg;
 			break;
 
 		case 'v':
@@ -292,7 +298,10 @@ int main(int argc, char **argv)
 			break;
 
 		case 'W':
-			radwtmp_path = optarg;
+			if (strcmp(optarg, "none") == 0)
+				use_wtmp = 0;
+			else
+				radwtmp_path = optarg;
 			break;
 
 		case 'w':
@@ -325,6 +334,8 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (optind < argc) usage();
+
 #ifdef USE_SYSLOG
 	if (strcmp(radlog_dir, "syslog") == 0 && syslog_facility < 0) {
 		fprintf(stderr,
@@ -664,10 +675,10 @@ int radrespond(AUTH_REQ *authreq, int activefd)
 	
 	case PW_PASSWORD_REQUEST:
 		/*
-		 *	FIXME: print an error message here.
 		 *	We don't support this anymore.
 		 */
-		/* rad_passchange(authreq, activefd); */
+		log(L_ERR, "Unsupported change password request packet "
+			   "from nas [%s]", client_name(authreq->ipaddr));
 		break;
 
 	case PW_STATUS_SERVER:
@@ -675,6 +686,8 @@ int radrespond(AUTH_REQ *authreq, int activefd)
 		break;
 
 	default:
+		log(L_ERR, "Unknown packet type [%d] from nas [%s]",
+			authreq->code, client_name(authreq->ipaddr));
 		break;
 	}
 
@@ -687,6 +700,8 @@ int radrespond(AUTH_REQ *authreq, int activefd)
 			rad_spawn_child(authreq, activefd, fun);
 		else
 			(*fun)(authreq, activefd);
+	} else {
+		authfree(authreq);
 	}
 
 	return 0;
