@@ -3,9 +3,9 @@
  *		used throughout the server.
  *
  *		Copyright 1996-2001	Cistron Internet Services B.V.
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  *
- * Version:	$Id: radiusd.h,v 1.31 2002/01/23 12:42:07 miquels Exp $
+ * Version:	$Id: radiusd.h,v 1.37 2003/07/02 09:39:03 miquels Exp $
  */
 
 #include "sysdep.h"
@@ -51,6 +51,7 @@ typedef struct value_pair {
 	int			length; /* of strvalue */
 	ATTR_FLAGS		flags;
 	UINT4			lvalue;
+	UINT4			lvalueh;
 	int			operator;
 	char			strvalue[AUTH_STRING_LEN];
 	int			group;
@@ -112,6 +113,7 @@ typedef struct realm_opts {
 	unsigned int		noauth:1;
 	unsigned int		noacct:1;
 	unsigned int		trusted:1;
+	unsigned int		wildcard:1;
 } REALM_OPTS;
 
 typedef struct realm {
@@ -206,6 +208,7 @@ extern int		radius_pid;
 extern int		use_dbm;
 extern int		use_dns;
 extern int		use_wtmp;
+extern int		use_utmp;
 extern int		log_stripped_names;
 extern int		cache_passwd;
 extern UINT4		myip;
@@ -225,9 +228,9 @@ extern int		acct_port;
 int		rad_add_detail(char *detail_file);
 int		rad_accounting(AUTH_REQ *, int);
 int		rad_accounting_detail(AUTH_REQ *, int, char *);
-int		radzap(UINT4 nas, int port, char *user, time_t t, int dowtmp);
+int		radzap(UINT4 nas, UINT4 port, char *user, time_t t, int dowtmp);
 char		*uue(void *);
-int		rad_check_multi(char *name, VALUE_PAIR *request, 
+int		rad_check_multi(char *name, AUTH_REQ *authreq, 
 			VALUE_PAIR *reply, int maxsimul);
 
 /* attrprint.c */
@@ -285,6 +288,7 @@ void		(*sun_signal(int signo, void (*func)(int)))(int);
 #define signal sun_signal
 #endif
 char		*strNcpy(char *dest, char *src, int n);
+int		wildmat(char *pat, char *str);
 
 /* radius.c */
 void		random_vector(unsigned char *vector);
@@ -310,7 +314,7 @@ int		huntgroup_access(VALUE_PAIR *request_pairs);
 RADCLIENT	*client_find(UINT4 ipno);
 char		*client_name(UINT4 ipno);
 int		read_clients_file(char *);
-REALM		*realm_find(char *, int);
+REALM		*realm_find(char *, char *, int);
 NAS		*nas_find(UINT4 ipno);
 char		*nas_name(UINT4 ipno);
 char		*nas_name2(AUTH_REQ *r);
