@@ -1,9 +1,10 @@
 /*
  * radutmp.h	Definitions for the Radius UTMP file.
  *
- *		Copyright 1998 Cistron Internet Services B.V.
+ *		Copyright 1998		Cistron Internet Services B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  *
- * Version:	$Id: radutmp.h,v 1.2 2000/12/01 16:13:23 miquels Exp $
+ * Version:	$Id: radutmp.h,v 1.5 2003/07/02 09:39:03 miquels Exp $
  */
 
 #ifndef _RADUTMP_H
@@ -35,9 +36,8 @@
 struct radutmp {
   char login[32];		/* Loginname */
 				/* FIXME: extend to 48 or 64 bytes */
-  int  nas_port;		/* Port on the terminal server (32 bits). */
-  char session_id[8];		/* Radius session ID (first 8 bytes at least)*/
-				/* FIXME: extend to 16 or 32 bytes */
+  unsigned int  nas_port;	/* Port on the terminal server (32 bits). */
+  char reserved[8];		/* 2 ints. Used to be session_id */
   unsigned int nas_address;	/* IP of portmaster. */
   unsigned int framed_address;	/* SLIP/PPP address or login-host. */
   int proto;			/* Protocol. */
@@ -47,7 +47,8 @@ struct radutmp {
   char porttype;		/* Porttype (I=ISDN A=Async T=Async-ISDN */
   char res1,res2,res3;		/* Fills up to one int */
   char caller_id[16];		/* Calling-Station-ID */
-  char reserved[12];		/* 3 ints reserved */
+  char session_id[12];		/* Radius session ID (first 12 bytes at least)*/
+				/* FIXME: extend to 16 or 32 bytes */
 };
 
 #endif /* _RADUTMP_H */
