@@ -3,17 +3,18 @@
  * version.c	Compiled in version info.
  *
  *		Copyright 1998-2001	Cistron Internet Services B.V.
- *		Copyright 2002		Cistron IP B.V.
+ *		Copyright 2002-2003	Cistron IP B.V.
  */
 
 char version_rcsid[] =
-"$Id: version.c,v 1.35 2002/02/06 15:23:16 miquels Exp $";
+"$Id: version.c,v 1.45 2003/07/03 08:09:32 miquels Exp $";
 
 #include        <sys/types.h>
 #include	<stdio.h>
+#include	<stdlib.h>
 #include	"radiusd.h"
 
-#define		VERSION		"1.6.6 06-Feb-2002"
+#define		VERSION		"1.6.7 21-Jul-2003"
 
 /*
  *	Display the revision number for this program
